
/*     *******************************************************************************
	XVisual GUI Development System for Linux/X-Window
	Copyright (C) 1996 Steve Carrie.

        This program is free software; you can redistribute it and/or modify
        it under the terms of the GNU General Public License as published by
        the Free Software Foundation; either version 2 of the License, or
        (at your option) any later version.

        This program is distributed in the hope that it will be useful,
        but WITHOUT ANY WARRANTY; without even the implied warranty of
        MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        GNU General Public License for more details.

        You should have received a copy of the GNU General Public License
        along with this program; if not, write to the Free Software
        Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
        ******************************************************************************* */

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <dirent.h>
#include <sys/stat.h>
#include <string.h>
#include <stdio.h>

#include <X11/Xaw3d/Viewport.h>
#include <X11/Xaw3d/List.h>
#include <X11/xv/xvCore.h>
#include <X11/xv/xvFont.h>
#include <X11/xv/xvView.h>
#include <X11/xv/xvList.h>
#include <X11/xv/xvVisualList.h>
#include <X11/xv/xvFileList.h>

// -------------------------------------------

xvFileList::xvFileList()
{
	numfiles=0;
	allocflag=FALSE;
	hiddenflag=FALSE;
}

// -------------------------------------------

xvFileList::xvFileList(String name, xvCore & parent, String path, int mode, Boolean hidden,
				XtCallbackProc filelist_callback, Boolean manage)
{
	numfiles=0;
	allocflag=FALSE;
	hiddenflag=FALSE;
	init(name, parent, path, mode, hidden, filelist_callback, manage);
}

// -------------------------------------------

xvFileList::~xvFileList()
{
	if (allocflag) {
		delete filelist;	
		delete superlist;
		allocflag=FALSE;
	}
}

// -------------------------------------------

void xvFileList::init(String name, xvCore & parent, String path, int mode, Boolean hidden,
				XtCallbackProc filelist_callback, Boolean manage)
{
	allocflag=FALSE;

	numfiles=0;
	numentries=0;

	NewPath(path, mode, hidden);

	xvView::init(name, parent.ID(), FALSE, FALSE, TRUE);
	xvList::init("xvFileList", xvView::ID(), filelist_callback, numfiles, filelist, 1);

	if (!manage) {
		XtTranslations trans=XtParseTranslationTable(" ");
		xvList::SetResource(XtNtranslations, (XtArgVal) trans);
	}
	xvList::SetWidth(150);
	xvList::SetHeight(180);	

	if (manage) xvView::ManageChildren();
}

// --------------------------------------------

int xvFileList::SetNewPath(String pathname, int mode, Boolean hidden)
{
	Dimension oldw=xvList::GetWidth();
	Dimension oldh=xvList::GetHeight();
	int vt=NewPath(pathname, mode, hidden);
	xvList::SetList(numfiles, filelist);
	xvView::SetWidth(oldw);
	xvView::SetHeight(oldh);

	return(vt);
}
	
// --------------------------------------------

int xvFileList::NewPath(String pathname, int mode, Boolean hidden)
{
	numentries=GetNumEntries(pathname);
	if (numentries >= 0) {
		datamode=mode;
		hiddenflag=hidden;
		strcpy(currentpath, pathname);
		if (allocflag==TRUE) {
			delete filelist;
			delete superlist;	
			allocflag=FALSE;
		}
		ParseFileTree(pathname);
	}
	return(numentries);
}

// ---------------------------------------------

int xvFileList::GetNumEntries(String pathname)
{
	DIR *dp;
	struct dirent *d;
	int cnt=0;

	dp=opendir(pathname);
	if (dp!=NULL) {
		while ((d=readdir(dp)) != NULL)
			cnt++;
		closedir(dp);
	}
	else
		cnt=-1;

	return(cnt);
}

// ------------------------------------------

int xvFileList::ParseFileTree(String pathname)
{
	DIR *dp;
	struct dirent *d;
	struct stat s;

	int listed=0;
	char buf[180];

	dp=opendir(pathname);
	if (dp!=NULL) {
		allocflag=TRUE;
		superlist=new struct dirent[numentries];
		filelist=new char*[numentries];

		for (int i=0; i<numentries; i++) 
			filelist[i]=NULL;
  
		numfiles=0;

		while ((d=readdir(dp)) != NULL) {
			strcpy(superlist[listed].d_name, d->d_name);
			strcpy(buf, pathname);
			strcat(buf, "/");
			strcat(buf, superlist[listed].d_name);
			stat(buf, &s);

			if ((datamode==XVFILELIST) || (datamode==XVALLLIST))
				if (S_ISDIR(s.st_mode) == 0) 
					if ((hiddenflag != 0) || (superlist[listed].d_name[0]!='.'))
						filelist[numfiles++]=superlist[listed].d_name;

			if ((datamode==XVDIRLIST) || (datamode==XVALLLIST))
				if (S_ISDIR(s.st_mode) != 0)
					if ((hiddenflag!=0) || (superlist[listed].d_name[0] != '.') || (superlist[listed].d_name[1]=='.')) {
						strcat(superlist[listed].d_name, "/");
						filelist[numfiles++]=superlist[listed].d_name;
					}
			listed++;
		}
		closedir(dp);
	}
	return(listed);
}

// --------------------------------------------------------					

void xvFileList::GetResource(String resname, XtArgVal *val)
{
	XtVaGetValues(xvList::ID(), resname, val, NULL);
}

// ------------------------------------------	

void xvFileList::SetResource(String resname, XtArgVal val)
{
	XtVaSetValues(xvList::ID(), resname, val, NULL);
}

// ------------------------------------------	

void xvFileList::SetFixedPosition(Boolean val)
{
	xvView::SetFixedPosition(val);
}

// ------------------------------------------

char *xvFileList::GetSelectedItem(XtPointer call, int pathmode)
{
	XawListReturnStruct *item=(XawListReturnStruct *) call;

	switch(pathmode) {
		case XVRELATIVEPATH:
			strcpy(workingbuffer, item->string);
			break;
		case XVABSOLUTEPATH:
			strcpy(workingbuffer, currentpath);
			strcat(workingbuffer, item->string);
			break;
		default:
			workingbuffer[0]='\0';
			break;
	}

	return(workingbuffer);
}

// ---------------------------------------------

char *xvFileList::Directory(String path)
{
	if (strcmp(path, "../")==0) {
		strcpy(dirbuffer, currentpath);
		int i=strlen(dirbuffer);
		if (i>=2) {
			i-=2;
			while ((dirbuffer[i]!='/') && (i>0)) i--;
			dirbuffer[i]='\0';
		}
		if (i<2)
			if (dirbuffer[0] !='/') strcpy(dirbuffer, "/");
	}
	else {
		strcpy(dirbuffer, currentpath);
		if (dirbuffer[strlen(dirbuffer)-1]!='/') strcat(dirbuffer, "/");
		strcat(dirbuffer, path);
	}
	return(dirbuffer);
}

// --------------------------------------------

void xvFileList::SetBackground(String name)
{
	xvList::SetBackground(name);
}

// --------------------------------------------

void xvFileList::SetBackground(Pixel bg)
{
	xvList::SetBackground(bg);
}

// --------------------------------------------

void xvFileList::SetForeground(String name)
{
	xvList::SetForeground(name);
}

// --------------------------------------------

void xvFileList::SetForeground(Pixel fg)
{
	xvList::SetForeground(fg);
}

// --------------------------------------------

void xvFileList::Map()
{
	xvView::Map();
}

// --------------------------------------------

void xvFileList::Unmap()
{
	xvView::Unmap();
}

// --------------------------------------------

void xvFileList::SetSensitive(Boolean sense)
{
	xvList::SetSensitive(sense);
}

// --------------------------------------------

void xvFileList::SetMappedWhenManaged(Boolean man)
{
	xvView::SetMappedWhenManaged(man);
}

// --------------------------------------------

void xvFileList::AddEvent(EventMask eventmask, XtEventHandler handler, XtPointer cdata)
{
	xvList::AddEvent(eventmask, handler, cdata);
}

// --------------------------------------------

void xvFileList::RemoveEvent(EventMask eventmask, XtEventHandler handler, XtPointer cdata)
{
	xvList::RemoveEvent(eventmask, handler, cdata);
}

// --------------------------------------------
